@@ -360,6 +360,12 @@ ldns_rdf_new_frm_str(ldns_rdf_type type, const char *str)
 	case LDNS_RDF_TYPE_MATCHING_TYPE:
 		status = ldns_str2rdf_matching_type(&rdf, str);
 		break;
+	case LDNS_RDF_TYPE_AMTRELAY:
+		status = ldns_str2rdf_amtrelay(&rdf, str);
+		break;
+	case LDNS_RDF_TYPE_SVCPARAMS:
+		status = ldns_str2rdf_svcparams(&rdf, str);
+		break;
 	case LDNS_RDF_TYPE_NONE:
 	default:
 		/* default default ??? */
